import express from 'express';
import multer from 'multer';
import multerS3 from 'multer-s3';
import { auth } from '../../middleware/auth';
import {createS3Client, s3Service} from '../../services/s3MulterService';
import { logger } from '../../config/logger';
import { imageResolutions } from '../../config/imageResolutions';

const router = express.Router();
//const upload = multer();

// Инициализация Multer-S3
const s3Client = createS3Client(); // Экспортируем клиент из s3Service
const upload = multer({
  storage: multerS3({
    s3: s3Client,
    bucket: process.env.AWS_S3_BUCKET || '',
    metadata: (req, file, cb) => {
      cb(null, { fieldName: file.fieldname });
    },
    key: (req, file, cb) => {
      const fileName = `${Date.now()}-${file.originalname}`;
      cb(null, `uploads/${fileName}`);
    },
    contentType: multerS3.AUTO_CONTENT_TYPE,
  }),
});

/*
router.post('/upload-url', auth, async (req, res) : Promise<void> => {
  try {
    const { fileName, fileType, resolution } = req.body;
    
    if (!fileName || !fileType || !resolution) {
      res.status(400).json({ error: 'Missing required fields' });
      return
    }
    
    const selectedResolution = imageResolutions.find(r => r.id === resolution);
    if (!selectedResolution) {
      res.status(400).json({ error: 'Invalid resolution' });
      return
    }

    const { uploadUrl, imageId, key } = await s3Service.getUploadUrl(fileName, fileType);
    
    logger.info(`Generated upload URL for image: ${fileName}`);
    res.json({ uploadUrl, imageId, key });
  } catch (error) {
    logger.error('Error generating upload URL:', error);
    res.status(500).json({ error: 'Failed to generate upload URL' });
  }
});
*/

// Маршрут для загрузки изображения
router.post('/upload-url', auth, upload.single('file'), async (req, res): Promise<void> => {
  try {
    const file = req.file as Express.MulterS3.File;
    const { resolutionId } = req.body;

    if (!file) {
      res.status(400).json({ error: 'Файл не найден' });
      return;
    }

    // Сохраняем оригинал и обрабатываем изображение
    const originalKey = await s3Service.uploadOriginalFile(file);
    const optimizedResult = await s3Service.optimizeAndUpload(file.buffer, originalKey, resolutionId);

    res.status(200).json({
      message: 'Файл успешно загружен и оптимизирован',
      originalKey,
      optimizedKey: optimizedResult.key,
      dimensions: {
        width: optimizedResult.width,
        height: optimizedResult.height,
      },
      format: optimizedResult.format,
      size: optimizedResult.size,
    });
  } catch (error) {
    logger.error('Ошибка загрузки изображения:', error);
    res.status(500).json({ error: 'Ошибка загрузки изображения' });
  }
});

/*
router.get('/:id', auth, async (req, res) => {
  try {
    const { id } = req.params;
    const image = await s3Service.getImage(id);
    res.json(image);
  } catch (error) {
    logger.error('Error fetching image:', error);
    res.status(500).json({ error: 'Failed to fetch image' });
  }
});
*/
export default router;